use warnings;
use strict;

use Test::More tests => 35;
use Test::Exception;

use POSIX;
use Time::HiRes qw( time );

use POWER::Utils::IO;


my (%r_fh, %w_fh, $r, $w, $timeout, %buf);
my ($fh);

throws_ok { Select()                    } qr/usage:/;
throws_ok { Select({})                  } qr/usage:/;
throws_ok { Select({},{},{},{})         } qr/usage:/;

my $ALARM = 0; $SIG{ALRM} = sub { $ALARM++ }; alarm(1);
Select({}, {});
is($ALARM, 1,                   'interrupted by signal');

my $t = time();
Select(undef, undef, 0.2);
cmp_ok($t+0.1, '<', time(),     'interrupted by timeout');
Select(undef, undef, 0);
cmp_ok($t+0.3, '>', time(),     'exit immediately with timeout 0');

throws_ok { Select({in=>undef}, undef)  } qr/is not open FILEHANDLE/;
throws_ok { Select(undef, {out=>0})     } qr/is not open FILEHANDLE/;

open $fh, '>', '/dev/null'  or die "open: $!";
POSIX::close(fileno($fh));
throws_ok { Select({fh=>$fh}, undef)    } qr/select: Bad file descriptor/;

open $fh, '>', '/dev/null'  or die "open: $!";
($r, $w, $timeout) = Select({null=>$fh}, {null=>$fh}, 0.1);
is_deeply($r, ['null'],         q{can   read from >/dev/null});
is_deeply($w, ['null'],         q{can   write to  >/dev/null});
cmp_ok($timeout, '>', 0,        q{io happens before timeout});
($r, $w, $timeout) = Select({null=>$fh}, {null=>$fh}, 0.1);

pipe my $rpipe, my $wpipe   or die "pipe: $!";
($r, $w, $timeout) = Select({pipe=>$rpipe}, {pipe=>$rpipe}, 0.1);
is_deeply($r, [],               q{can't read from <rpipe});
is_deeply($w, [],               q{can't write to  <rpipe});
($r, $w, $timeout) = Select({pipe=>$wpipe}, {pipe=>$wpipe}, 0.1);
is_deeply($r, [],               q{can't read from >wpipe});
is_deeply($w, ['pipe'],         q{can   write to  >wpipe});
is(syswrite($wpipe, 'x'), 1,    q{1 byte sent to  >wpipe});
($r, $w, $timeout) = Select({pipe=>$rpipe}, {pipe=>$rpipe}, 0.1);
is_deeply($r, ['pipe'],         q{can   read from <rpipe});
is_deeply($w, [],               q{can't write to  <rpipe});
is(sysread($rpipe, my $x, 5), 1,q{got 1 byte from <rpipe});
is($x, 'x',                     q{  data correct});

my $PIPE = 0; $SIG{PIPE} = sub { $PIPE++ };
($r, $w, $timeout) = Select({pipe=>$wpipe}, {pipe=>$wpipe}, 0.1);
is_deeply($r, [],               q{can't read from >wpipe});
is_deeply($w, ['pipe'],         q{can   write to  >wpipe});
is(syswrite($wpipe, 'x'), 1,    q{1 byte sent to  >wpipe});
ok(close($rpipe),               q{closed          <rpipe});
($r, $w, $timeout) = Select(undef,          {pipe=>$wpipe}, 0.1);
is_deeply($r, [],               q{can't read from >wpipe!});
is_deeply($w, ['pipe'],         q{can   write to  >wpipe});
($r, $w, $timeout) = Select({pipe=>$wpipe}, {pipe=>$wpipe}, 0.1);
is_deeply($r, ['pipe'],         q{can   read from >wpipe!});
is_deeply($w, ['pipe'],         q{can   write to  >wpipe});
{ no warnings 'io';
is(sysread($wpipe, $x, 5),undef,q{read error from >wpipe});
}
like($!,qr/Bad file descriptor/,q{  Bad file descriptor});
is($PIPE, 0,                    q{no SIGPIPE yet});
is(syswrite($wpipe, 'x'), undef,q{write error to  >wpipe});
like($!,qr/Broken pipe/,        q{  Broken pipe});
is($PIPE, 1,                    q{got SIGPIPE});

